// Camera - SX170IS - platform_camera.h

// This file contains the various settings values specific to the SX170IS camera.
// This file is referenced via the 'include/camera.h' file and should not be loaded directly.

// If adding a new settings value put a suitable default in 'include/camera.h',
// along with documentation on what the setting does and how to determine the correct value.
// If the setting should not have a default value then add it in 'include/camera.h'
// using the '#undef' directive along with appropriate documentation.

// Override any default values with your camera specific values in this file. Try and avoid
// having override values that are the same as the default value.

// When overriding a setting value there are two cases:
// 1. If removing the value, because it does not apply to your camera, use the '#undef' directive.
// 2. If changing the value it is best to use an '#undef' directive to remove the default value
//    followed by a '#define' to set the new value.

// When porting CHDK to a new camera, check the documentation in 'include/camera.h'
// for information on each setting. If the default values are correct for your camera then
// don't override them again in here.


    #define CAM_PROPSET                     6
    #define CAM_DRYOS                       1
    #define CAM_DRYOS_2_3_R39               1
    #define CAM_DRYOS_2_3_R47               1

    #undef CAM_USE_ZOOM_FOR_MF

    #undef CAM_DEFAULT_ALT_BUTTON
    #define CAM_DEFAULT_ALT_BUTTON          KEY_PLAYBACK
    #define CAM_ADJUSTABLE_ALT_BUTTON       1
    #define CAM_ALT_BUTTON_NAMES            { "Playback", "Disp", "Video" }
    #define CAM_ALT_BUTTON_OPTIONS          { KEY_PLAYBACK, KEY_DISPLAY, KEY_VIDEO }

    // TODO:
    #define CAM_SD_OVER_IN_AF               1
    #define CAM_SD_OVER_IN_AFL              1
    #define CAM_SD_OVER_IN_MF               1

    #undef CAM_CAN_UNLOCK_OPTICAL_ZOOM_IN_VIDEO  // unlocked in canon firmware by default

    #define CAM_HAS_JOGDIAL                 1
    #define CAM_HAS_VIDEO_BUTTON            1
    #undef CAM_CHDK_HAS_EXT_VIDEO_MENU           // not working

    #define CAM_HAS_FILEWRITETASK_HOOK      1 // TODO
    #define CAM_FILEWRITETASK_SEEKS         1

    #undef  CAM_UNCACHED_BIT
    #define CAM_UNCACHED_BIT                0x40000000 // Found @0xff815f04

    #undef CAM_SENSOR_BITS_PER_PIXEL
    #define CAM_SENSOR_BITS_PER_PIXEL       12

    #undef  CAM_USES_ASPECT_CORRECTION
    #define CAM_USES_ASPECT_CORRECTION      1
    #undef  CAM_BITMAP_WIDTH
    #define CAM_BITMAP_WIDTH                720

    #define CAM_ZEBRA_NOBUF                 1

    #define CAM_DATE_FOLDER_NAMING          0x080 // Found @0xffa9ecbc (pass as 3rd param to GetImageFolder)

    #define CAM_RAW_ROWPIX                  4704 // Found @0xff959c2c
    #define CAM_RAW_ROWS                    3504 // Found @0xff959c38
    #define CAM_JPEG_WIDTH                  4608 // From Canon specs
    #define CAM_JPEG_HEIGHT                 3456 // From Canon specs

    // TODO:
    #define CAM_ACTIVE_AREA_X1              0
    #define CAM_ACTIVE_AREA_Y1              0
    #define CAM_ACTIVE_AREA_X2              4704 // active all the way to the edge
    #define CAM_ACTIVE_AREA_Y2              3504 // active all the way to the bottom

    #define cam_CFAPattern                  0x01000201 // [Green Blue Red Green]
    //TODO:
    #define CAM_COLORMATRIX1                             \
        827547, 1000000, -290458, 1000000, -126086, 1000000, \
        -12829, 1000000,  530507, 1000000,   50537, 1000000, \
          5181, 1000000,   48183, 1000000,  245014, 1000000
    #define cam_CalibrationIlluminant1      1       // Daylight

    #define CAM_DNG_LENS_INFO               { 50,10, 800,10, 35,10, 59,10 }

    #define PARAM_CAMERA_NAME               3 // TODO

    #undef CAM_USB_EVENTID
    #define CAM_USB_EVENTID                 0x202

    #define DRAW_ON_ACTIVE_BITMAP_BUFFER_ONLY       1 // TODO

    // MakeDirectory_Fut not found, using mkdir
    #define MKDIR_RETURN_ONE_ON_SUCCESS     1

    #undef CAM_AF_LED
    #define CAM_AF_LED                      1

    #define CAM_FILE_COUNTER_IS_VAR         1 // TODO
